@@ -139,7 +139,7 @@ class FinalRestDoorKey(Choice):
     
     Postcards: Turn in an amount of postcards in Traverse Town
     
-    Final Ansem: Enter End of the World and defeat Ansem as normal
+    Lucky Emblems: Collect an amount of Lucky Emblems
     
     Puppies: Rescue and return an amount of puppies in Traverse Town
     
@@ -214,11 +214,9 @@ class EXPMultiplier(NamedRange):
     """
     display_name = "EXP Multiplier"
     default = 16 * 4
-    range_start = 16 // 4
+    range_start = 16
     range_end = 128
     special_range_names = {
-        "0.25x": int(16 // 4),
-        "0.5x": int(16 // 2),
         "1x": 16,
         "2x": 16 * 2,
         "3x": 16 * 3,
@@ -257,7 +255,7 @@ class KeybladeStats(Choice):
     """
     Determines whether Keyblade stats should be randomized.
     
-    Randomize: Randomly generates STR and MP bonuses for each keyblade between the defined minimums and maximums.
+    Randomize: Randomly generates stats for each keyblade between the defined minimums and maximums.
     
     Shuffle: Shuffles the stats of the vanilla keyblades amongst each other.
     
@@ -308,7 +306,7 @@ class KeybladeMinCritSTRBonus(Range):
     """
     Determines the minimum Crit STR bonus a keyblade can have.
     """
-    display_name = "Keyblade Minimum Crit Rate Bonus"
+    display_name = "Keyblade Minimum Crit STR Bonus"
     default = 0
     range_start = 0
     range_end = 16
@@ -317,7 +315,7 @@ class KeybladeMaxCritSTRBonus(Range):
     """
     Determines the maximum Crit STR bonus a keyblade can have.
     """
-    display_name = "Keyblade Maximum Crit Rate Bonus"
+    display_name = "Keyblade Maximum Crit STR Bonus"
     default = 16
     range_start = 0
     range_end = 16
@@ -513,8 +511,6 @@ class StartingWorlds(Range):
     Number of random worlds to start with in addition to Traverse Town, which is always available.
     
     Will only consider Atlantica if toggled, and will only consider End of the World if its unlock is set to "Item".
-    
-    These are given by the server, and are received after connection.
     """
     display_name = "Starting Worlds"
     default = 4
@@ -524,8 +520,6 @@ class StartingWorlds(Range):
 class StartingTools(DefaultOnToggle):
     """
     Determines whether you start with Scan and Dodge Roll.
-    
-    These are given by the server, and are received after connection.
     """
     display_name = "Starting Tools"
 
@@ -547,9 +541,9 @@ class RemoteItems(Choice):
 
 class Slot2LevelChecks(Range):
     """
-    Determines how many levels have an additional item.  Usually, this item is an ability.
+    Determines how many levels have an additional item.
     
-    If Remote Items is OFF, these checks will only contain abilities.
+    If Remote Items is OFF, these checks will only contain abilities or items for other players.
     """
     display_name = "Slot 2 Level Checks"
     default = 0
@@ -589,7 +583,7 @@ class OrichalcumInPool(Range):
     
     You need 17 to synth every recipe that requires it.
     """
-    display_name = "Mythril In Pool"
+    display_name = "Orichalcum In Pool"
     default = 20
     range_start = 17
     range_end = 30
@@ -761,7 +755,7 @@ class HomecomingMaterials(Range):
 
 class MaterialsInPool(Range):
     """
-    The amount of Raft Materials required to access Homecoming.
+    The amount of Raft Materials that will be added to the item pool.
     """
     display_name = "Materials in Pool"
     default = 16
@@ -796,6 +790,55 @@ class HalloweenTownKeyItemBundle(DefaultOnToggle):
     """
     display_name = "Halloween Town Key Item Bundle"
 
+class RandomizeSpellMPCosts(Choice):
+    """
+    Off: No randomization.
+    Shuffle: Spell costs will be shuffled amongst themselves, (2 single pip spells, 3 singe MP spells, 2 double MP spells)
+    Randomize: Spell costs will be randomized individually based on the defined upper and lower bounds.
+    """
+    display_name = "Randomize Spell MP Costs"
+    option_off = 0
+    option_shuffle = 1
+    option_randomize = 2
+    default = 0
+
+class SpellMPCostMin(Choice):
+    """
+    Define the minimum MP cost a spell can have
+    """
+    display_name = "Spell MP Cost Minimum"
+    option_half_pip = 15
+    option_pip = 30
+    option_1_mp = 100
+    option_2_mp = 200
+    option_3_mp = 300
+    default = 15
+    
+class SpellMPCostMax(Choice):
+    """
+    Define the maximum MP cost a spell can have
+    """
+    display_name = "Spell MP Cost Maximum"
+    option_half_pip = 15
+    option_pip = 30
+    option_1_mp = 100
+    option_2_mp = 200
+    option_3_mp = 300
+    default = 300
+
+class IndividualSpellLevelCosts(Toggle):
+    """
+    If randomizing or shuffling spell MP costs, each indiviudal level of a spell (for example Fire vs Fira vs Firaga)
+    can have different MP costs.
+    """
+    display_name = "Individal Spell Level Costs"
+    
+class ScalingSpellPotency(DefaultOnToggle):
+    """
+    If randomizing or shuffling spell MP costs, each spell will have its effectiveness scaled relative to its new cost.
+    """
+    display_name = "Scaling Spell Potency"
+
 @dataclass
 class KH1Options(PerGameCommonOptions):
     final_rest_door_key: FinalRestDoorKey
@@ -877,6 +920,11 @@ class KH1Options(PerGameCommonOptions):
     materials_in_pool: MaterialsInPool
     stacking_world_items: StackingWorldItems
     halloween_town_key_item_bundle: HalloweenTownKeyItemBundle
+    randomize_spell_mp_costs: RandomizeSpellMPCosts
+    spell_mp_cost_min: SpellMPCostMin
+    spell_mp_cost_max: SpellMPCostMax
+    individual_spell_level_costs: IndividualSpellLevelCosts
+    scaling_spell_potency: ScalingSpellPotency
     
 
 kh1_option_groups = [
@@ -939,8 +987,15 @@ kh1_option_groups = [
     ]),
     OptionGroup("AP Costs", [
         RandomizeAPCosts,
-        MaxAPCost,
-        MinAPCost
+        MinAPCost,
+        MaxAPCost
+    ]),
+    OptionGroup("Spells", [
+        RandomizeSpellMPCosts,
+        SpellMPCostMin,
+        SpellMPCostMax,
+        IndividualSpellLevelCosts,
+        ScalingSpellPotency
     ]),
     OptionGroup("Misc", [
         StartingWorlds,
